from enum import Enum
from typing import Any


class ResultCase:
    SUCCESS = 'success'
    ERROR = 'error'


class Result(object):
    __slots__ = ('state', 'case', 'message')
    
    def __init__(self, state: dict, case: str) -> None:
        self.state = state
        self.case = case
        self.message = None
        
    def __repr__(self) -> str:
        return f'Result(state={self.state}, case={self.case}, message={self.message})'
    
    def is_success(self) -> bool:
        return self.case == ResultCase.SUCCESS
    
    def is_failed(self) -> bool:
        return self.case == ResultCase.ERROR
    
    @property
    def failure_reason(self):
        if self.is_failed():
            return self.message


from typing import Any

from contractsPY.exceptions import StateException


class State:
    __dict = {}
    
    def __repr__(self) -> str:
        return str(self.__dict)
    
    def __setattr__(self, __name: str, __value: Any) -> None:
        self.__dict[__name] = __value
        
    def __getattr__(self, __name: str) -> Any:
        if self.__dict.get(__name):
            return self.__dict[__name]
        raise StateException(f'{__name} is not set')
    
    def __str__(self) -> str:
        return str(self.__dict)


from contractsPY.result import ResultCase, Result
 
 
def if_fails(message):
    def wrapper(func):
        setattr(func, 'message', message)
        return func
    return wrapper
    

def chained(state):
    def wrapper(func):
        try:
            expected_result = func(state)
            if not expected_result:
                result = Result(state, case=ResultCase.ERROR)
                result.message = expected_result.message
                return result
            
            return Result(
                state,
                case=ResultCase.SUCCESS
            )
        except:
            result = Result(state, case=ResultCase.ERROR)
            result.message = func.message
            return result
    return wrapper
    


from typing import Any

from contractsPY.decorators import chained
from contractsPY.state import State
from contractsPY.result import Result, ResultCase
from contractsPY.exceptions import StateException, ContractException


class Usecase:
    """
        Main class for usecase.
        
        Main methods:
            contract - list of functions that will be executed in order
            apply - execute contract functions
    """
    __state = State()
    __contract = []
    
    @property
    def state(self) -> State:
        return self.__state
    
    @state.setter
    def state(self, values: dict) -> None:
        
        if not isinstance(values, dict):
            raise Exception('State must be a dict')
        
        for k, v in values.items():
            setattr(self.__state, k, v)
        
    @property
    def contract(self) -> list:
        return self.__contract
    
    @contract.setter
    def contract(self, funcs: list) -> None:
        
        if not funcs:
            raise ContractException('Contract cannot be empty')
        
        if len(self.contract) > 0:
            raise ContractException('Contract cannot be changed')
        
        if not isinstance(funcs, list):
            raise ContractException('Contract must be a list')
        
        for func in funcs:
            
            if not callable(func):
                raise ContractException('Contract must be a list of callables')
            
            self.__contract.append(func)
    
    def apply(self, **kwargs) -> Result:
        self.state = kwargs
        
        for func in self.contract:
           
            result = chained(self.state)(func)
            if not result.is_success():
                return result
                
        return Result(self.state, ResultCase.SUCCESS)


import pathlib
from setuptools import setup
  
  

HERE = pathlib.Path(__file__).parent


README = (HERE / "README.md").read_text()
  
  
# There is no any requirements.
REQUIREMENTS = []
  
# some more details
CLASSIFIERS = [
    'Intended Audience :: Developers',
    'Topic :: Internet',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    ]
  

setup(
    name='contractsPY',
    version='1.0.2',
    description='Python Business Transactions Library',
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/arzuhuseyn/contractsPY',
    author='Arzu Huseynov',
    author_email='hi@arzuh.me',
    license='MIT',
    packages=['contractsPY'],
    classifiers=CLASSIFIERS,
    install_requires=REQUIREMENTS,
    keywords='railway-oriented usecases transactions contracts',
)